/* 
 * CDDL HEADER START
 * The contents of this file are subject to the terms
 * of the Common Development and Distribution License 
 * (the License). You may not use this file except in
 * compliance with the License.
 *
 * You can obtain a copy of the License at
 * http://www.sun.com/cddl/cddl.html and legal/CDDLv1.0.txt
 * See the License for the specific language governing
 * permission and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL 
 * Header Notice in each file and include the License file  
 * at legal/CDDLv1.0.txt.                                                           
 * If applicable, add the following below the CDDL Header,
 * with the fields enclosed by brackets [] replaced by
 * your own identifying information: 
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Copyright 2006 Sun Microsystems Inc. All Rights Reserved
 * CDDL HEADER END
*/

package com.sun.faces.portlet;


import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.security.Principal;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Set;
import java.util.AbstractMap;

import javax.faces.FacesException;
import javax.faces.context.ExternalContext;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.PortletContext;
import javax.portlet.PortletException;
import javax.portlet.PortletRequest;
import javax.portlet.PortletRequestDispatcher;
import javax.portlet.PortletResponse;
import javax.portlet.PortletSession;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


/**
 * <p>Concrete implementation of <code>ExternalContext</code> for use in
 * a portlet environment.</p>
 */

public class ExternalContextImpl extends ExternalContext {
    
    
    // ------------------------------------------------------------ Constructors
    
    
    /**
     * <p>Create a new instance wrapping the specified Portlet API objects.</p>
     *
     * @param context The <code>PortletContext</code> for this application
     * @param request The <code>PortletRequest</code> for this request
     * @param response The <code>PortletResponse</code> for this response
     */
    public ExternalContextImpl(PortletContext context,
            PortletRequest request,
            PortletResponse response) {
        this.context = context;
        this.request = request;
        this.response = response;
        if (log.isTraceEnabled()) {
            log.trace("Created ExternalContext " + this);
        }
    }
    
    
    // ------------------------------------------------------ Instance Variables
    
    
    // The Log instance for this class
    private static final Log log = LogFactory.getLog(ExternalContextImpl.class);
    
    
    // The Portlet API objects we are wrapping
    private PortletContext context;
    private PortletRequest request;
    private PortletResponse response;
    
    
    // Our private map implementations (lazily instantiated)
    private PortletApplicationMap applicationMap = null;
    private PortletInitParameterMap initParameterMap = null;
    private PortletRequestHeaderMap requestHeaderMap = null;
    private PortletRequestHeaderValuesMap requestHeaderValuesMap = null;
    private PortletRequestMap requestMap = null;
    private PortletRequestParameterMap requestParameterMap = null;
    private PortletRequestParameterValuesMap requestParameterValuesMap = null;
    private PortletSessionMap sessionMap = null;
    
    
    // ---------------------------------------------------------- Public Methods
    
    
    public void dispatch(String path) throws IOException {
        if (log.isTraceEnabled()) {
            log.trace("dispatchMessage(" + path + ")");
        }
        if (path == null) {
            throw new NullPointerException();
        }
        if (!(request instanceof RenderRequest) ||
                !(response instanceof RenderResponse)) {
            throw new IllegalStateException(path);
        }
        if (log.isDebugEnabled()) {
            log.debug("response content type " + ((RenderResponse)response).getContentType());
        }
        PortletRequestDispatcher prd = context.getRequestDispatcher(path);
        if (prd == null) {
            throw new IllegalArgumentException(path);
        }
        try {
            prd.include((RenderRequest) request, (RenderResponse) response);
        } catch (PortletException e) {
            throw new FacesException(e);
        }
    }
    
    public String encodeActionURL(String url) {
        if (url == null) {
            throw new NullPointerException();
        }
        // fix for bug 6243708
        // wrong usage of  PortletResponse.encodeURL
        // method. This method is to be only used to
        // "encode" resource URLs (i.e. servlets, JSPs,
        // static files such as images, etc) and not portlet
        // action or render URLs.
        // return (response.encodeURL(url));
        return url;
    }
    
    
    public String encodeNamespace(String name) {
        if (!(response instanceof RenderResponse)) {
            throw new IllegalStateException();
        }
        return (((RenderResponse) response).getNamespace() + name);
    }
    
    
    // PENDING(craigmcc) - Currently identical to encodeActionURL()
    public String encodeResourceURL(String url) {
        if (url == null) {
            throw new NullPointerException();
        }
        return (response.encodeURL(url));
    }
    
    
    public Map getApplicationMap() {
        if (applicationMap == null) {
            applicationMap = new PortletApplicationMap(context);
        }
        return (applicationMap);
    }
    
    
    public String getAuthType() {
        return (request.getAuthType());
    }
    
    
    public Object getContext() {
        return (context);
    }
    
    
    public String getInitParameter(String name) {
        if (name == null) {
            throw new NullPointerException();
        }
        return (context.getInitParameter(name));
    }
    
    
    public Map getInitParameterMap() {
        if (initParameterMap == null) {
            initParameterMap = new PortletInitParameterMap(context);
        }
        return (initParameterMap);
    }
    
    
    public String getRemoteUser() {
        return (request.getRemoteUser());
    }
    
    
    public Object getRequest() {
        return (request);
    }
    
    
    public String getRequestContextPath() {
        return (request.getContextPath());
    }
    
    
    // PENDING(craigmcc) - Do we want to try to parse "Cookie" properties?
    public Map getRequestCookieMap() {
        return (Collections.unmodifiableMap(Collections.EMPTY_MAP));
    }
    
    
    public Map getRequestHeaderMap() {
        if (requestHeaderMap == null) {
            requestHeaderMap = new PortletRequestHeaderMap(request);
        }
        return (requestHeaderMap);
    }
    
    
    public Map getRequestHeaderValuesMap() {
        if (requestHeaderValuesMap == null) {
            requestHeaderValuesMap = new PortletRequestHeaderValuesMap(request);
        }
        return (requestHeaderValuesMap);
    }
    
    
    public Locale getRequestLocale() {
        return (request.getLocale());
    }
    
    
    public Iterator getRequestLocales() {
        return (new LocalesIterator(request.getLocales()));
    }
    
    
    public Map getRequestMap() {
        if (requestMap == null) {
            requestMap = new PortletRequestMap(request);
        }
        return (requestMap);
    }
    
    
    public Map getRequestParameterMap() {
        if (requestParameterMap == null) {
            requestParameterMap = new PortletRequestParameterMap(request);
        }
        return (requestParameterMap);
    }
    
    
    public Iterator getRequestParameterNames() {
        return (request.getParameterMap().keySet().iterator());
    }
    
    
    public Map getRequestParameterValuesMap() {
        if (requestParameterValuesMap == null) {
            requestParameterValuesMap = new PortletRequestParameterValuesMap(request);
        }
        return (requestParameterValuesMap);
    }
    
    
    // PENDING(craigmcc) - Alternate interpretation in portlet environment?
    public String getRequestPathInfo() {
        return (null);
    }
    
    
    // PENDING(craigmcc) - Alternate interpretation in portlet environment?
    public String getRequestServletPath() {
        return (null);
    }
    
    
    public URL getResource(String path) throws MalformedURLException {
        if (path == null) {
            throw new NullPointerException();
        }
        return (context.getResource(path));
    }
    
    
    public InputStream getResourceAsStream(String path) {
        if (path == null) {
            throw new NullPointerException();
        }
        return (context.getResourceAsStream(path));
    }
    
    
    
    public Set getResourcePaths(String path) {
        if (path == null) {
            throw new NullPointerException();
        }
        return (context.getResourcePaths(path));
    }
    
    
    public Object getResponse() {
        return (response);
    }
    
    
    public Object getSession(boolean create) {
        return (new PortletSessionWrapper(request.getPortletSession(create),
                response, getRequestParameterMap()));
    }
    
    
    public Map getSessionMap() {
        if (sessionMap == null) {
            sessionMap = new PortletSessionMap(request, response, getRequestParameterMap());
        }
        return (sessionMap);
    }
    
    
    public Principal getUserPrincipal() {
        return (request.getUserPrincipal());
    }
    
    
    public boolean isUserInRole(String role) {
        if (role == null) {
            throw new NullPointerException();
        }
        return (request.isUserInRole(role));
    }
    
    
    public void log(String message) {
        if (message == null) {
            throw new NullPointerException();
        }
        context.log(message);
    }
    
    
    public void log(String message, Throwable exception) {
        if ((message == null) || (exception == null)) {
            throw new NullPointerException();
        }
        context.log(message, exception);
    }
    
    public void redirect(String path) throws IOException {
        if (log.isTraceEnabled()) {
            log.trace("redirectMessage(" + path + ")");
        }
        if (path == null) {
            throw new NullPointerException();
        }
        if (!(request instanceof ActionRequest) ||
                !(response instanceof ActionResponse)) {
            throw new IllegalStateException(path);
        }
        try {
            ((ActionResponse) response).sendRedirect(path);
        } catch (Exception e) {
            throw new FacesException(e);
        }
    }
    
}
// ------------------------------------------------------------- Private Classes


class LocalesIterator implements Iterator {
    
    public LocalesIterator(Enumeration locales) {
        this.locales = locales;
    }
    private Enumeration locales;
    
    public boolean hasNext() { return locales.hasMoreElements(); }
    public Object next() { return locales.nextElement(); }
    public void remove() { throw new UnsupportedOperationException(); }
    
}

abstract class BaseContextMap extends AbstractMap{
    
    // Unsupported by all Maps.
    public void clear() {
        throw new UnsupportedOperationException();
    }
    
    
    // Unsupported by all Maps.
    public void putAll(Map t) {
        throw new UnsupportedOperationException();
    }
    
    
    // Supported by maps if overridden
    public Object remove(Object key) {
        throw new UnsupportedOperationException();
    }
    
    
    static class Entry implements Map.Entry {
        
        // immutable Entry
        private final Object key;
        private final Object value;
        
        
        Entry(Object key, Object value) {
            this.key = key;
            this.value = value;
        }
        
        
        public Object getKey() {
            return key;
        }
        
        
        public Object getValue() {
            return value;
        }
        
        
        // No support of setting the value
        public Object setValue(Object value) {
            throw new UnsupportedOperationException();
        }
        
        
        public int hashCode() {
            return ((key == null ? 0 : key.hashCode()) ^
                    (value == null ? 0 : value.hashCode()));
        }
        
        
        public boolean equals(Object obj) {
            if (obj == null || !(obj instanceof Map.Entry))
                return false;
            
            Map.Entry input = (Map.Entry) obj;
            Object inputKey = input.getKey();
            Object inputValue = input.getValue();
            
            if (inputKey == key ||
                    (inputKey != null && inputKey.equals(key))) {
                if (inputValue == value ||
                        (inputValue != null && inputValue.equals(value))) {
                    return true;
                }
            }
            return false;
        }
    }
}

class PortletApplicationMap extends BaseContextMap {
    
    
    public PortletApplicationMap(PortletContext context) {
        this.context = context;
    }
    
    
    private PortletContext context = null;
    
    
    public void clear() {
        Iterator keys = keySet().iterator();
        while (keys.hasNext()) {
            context.removeAttribute((String) keys.next());
        }
    }
    
    
    public boolean containsKey(Object key) {
        return (context.getAttribute(key(key)) != null);
    }
    
    
    public boolean containsValue(Object value) {
        if (value == null) {
            return (false);
        }
        Enumeration keys = context.getAttributeNames();
        while (keys.hasMoreElements()) {
            Object next = context.getAttribute((String) keys.nextElement());
            if (next == value) {
                return (true);
            }
        }
        return (false);
    }
    
    public Set entrySet() {
        Set entries = new HashSet();
        for (Enumeration e = context.getAttributeNames();
        e.hasMoreElements();) {
            String key = (String) e.nextElement();
            entries.add(new Entry(key, context.getAttribute(key)));
        }
        return entries;
    }
    
    
    public boolean equals(Object o) {
        return (context.equals(o));
    }
    
    
    public Object get(Object key) {
        return (context.getAttribute(key(key)));
    }
    
    
    public int hashCode() {
        return (context.hashCode());
    }
    
    
    public boolean isEmpty() {
        return (size() < 1);
    }
    
    
    public Set keySet() {
        Set set = new HashSet();
        Enumeration keys = context.getAttributeNames();
        while (keys.hasMoreElements()) {
            set.add(keys.nextElement());
        }
        return (set);
    }
    
    
    public Object put(Object key, Object value) {
        if (value == null) {
            return (remove(key));
        }
        String skey = key(key);
        Object previous = context.getAttribute(skey);
        context.setAttribute(skey, value);
        return (previous);
    }
    
    
    public void putAll(Map map) {
        Iterator keys = map.keySet().iterator();
        while (keys.hasNext()) {
            String key = (String) keys.next();
            context.setAttribute(key, map.get(key));
        }
    }
    
    
    public Object remove(Object key) {
        String skey = key(key);
        Object previous = context.getAttribute(skey);
        context.removeAttribute(skey);
        return (previous);
    }
    
    
    public int size() {
        int n = 0;
        Enumeration keys = context.getAttributeNames();
        while (keys.hasMoreElements()) {
            keys.nextElement();
            n++;
        }
        return (n);
    }
    
    
    public Collection values() {
        List list = new ArrayList();
        Enumeration keys = context.getAttributeNames();
        while (keys.hasMoreElements()) {
            list.add(context.getAttribute((String) keys.nextElement()));
        }
        return (list);
    }
    
    
    private String key(Object key) {
        if (key == null) {
            throw new IllegalArgumentException();
        } else if (key instanceof String) {
            return ((String) key);
        } else {
            return (key.toString());
        }
    }
    
    
}


class PortletInitParameterMap extends BaseContextMap {
    
    
    public PortletInitParameterMap(PortletContext context) {
        this.context = context;
    }
    
    
    private PortletContext context = null;
    
    
    public void clear() {
        throw new UnsupportedOperationException();
    }
    
    
    public boolean containsKey(Object key) {
        return (context.getInitParameter(key(key)) != null);
    }
    
    
    public boolean containsValue(Object value) {
        Iterator values = values().iterator();
        while (values.hasNext()) {
            if (value.equals(values.next())) {
                return (true);
            }
        }
        return (false);
    }
    
    public Set entrySet() {
        Set entries = new HashSet();
        
        for (Enumeration e = context.getInitParameterNames();
        e.hasMoreElements();) {
            String initParamName = (String) e.nextElement();
            entries.add(new Entry(initParamName,
                    context.getInitParameter(
                    initParamName)));
        }
        return entries;
    }
    
    
    public boolean equals(Object o) {
        return (context.equals(o));
    }
    
    
    public Object get(Object key) {
        return (context.getInitParameter(key(key)));
    }
    
    
    public int hashCode() {
        return (context.hashCode());
    }
    
    
    public boolean isEmpty() {
        return (size() < 1);
    }
    
    
    public Set keySet() {
        Set set = new HashSet();
        Enumeration keys = context.getInitParameterNames();
        while (keys.hasMoreElements()) {
            set.add(keys.nextElement());
        }
        return (set);
    }
    
    
    public Object put(Object key, Object value) {
        throw new UnsupportedOperationException();
    }
    
    
    public void putAll(Map map) {
        throw new UnsupportedOperationException();
    }
    
    
    public Object remove(Object key) {
        throw new UnsupportedOperationException();
    }
    
    
    public int size() {
        int n = 0;
        Enumeration keys = context.getInitParameterNames();
        while (keys.hasMoreElements()) {
            keys.nextElement();
            n++;
        }
        return (n);
    }
    
    
    public Collection values() {
        List list = new ArrayList();
        Enumeration keys = context.getInitParameterNames();
        while (keys.hasMoreElements()) {
            list.add(context.getInitParameter((String) keys.nextElement()));
        }
        return (list);
    }
    
    
    private String key(Object key) {
        if (key == null) {
            throw new IllegalArgumentException();
        } else if (key instanceof String) {
            return ((String) key);
        } else {
            return (key.toString());
        }
    }
    
    
}


class PortletRequestHeaderMap extends BaseContextMap {
    
    
    public PortletRequestHeaderMap(PortletRequest request) {
        this.request = request;
    }
    
    
    private PortletRequest request = null;
    
    
    public void clear() {
        throw new UnsupportedOperationException();
    }
    
    
    public boolean containsKey(Object key) {
        return (request.getProperty(key(key)) != null);
    }
    
    
    public boolean containsValue(Object value) {
        Iterator values = values().iterator();
        while (values.hasNext()) {
            if (value.equals(values.next())) {
                return (true);
            }
        }
        return (false);
    }
    
    
    public Set entrySet() {
        Set set = new HashSet();
        Enumeration keys = request.getPropertyNames();
        while (keys.hasMoreElements()) {
            set.add(request.getProperty((String) keys.nextElement()));
        }
        return (set);
    }
    
    public boolean equals(Object o) {
        return (request.equals(o));
    }
    
    
    public Object get(Object key) {
        return (request.getProperty(key(key)));
    }
    
    
    public int hashCode() {
        return (request.hashCode());
    }
    
    
    public boolean isEmpty() {
        return (size() < 1);
    }
    
    
    public Set keySet() {
        Set set = new HashSet();
        Enumeration keys = request.getPropertyNames();
        while (keys.hasMoreElements()) {
            set.add(keys.nextElement());
        }
        return (set);
    }
    
    
    public Object put(Object key, Object value) {
        throw new UnsupportedOperationException();
    }
    
    
    public void putAll(Map map) {
        throw new UnsupportedOperationException();
    }
    
    
    public Object remove(Object key) {
        throw new UnsupportedOperationException();
    }
    
    
    public int size() {
        int n = 0;
        Enumeration keys = request.getPropertyNames();
        while (keys.hasMoreElements()) {
            keys.nextElement();
            n++;
        }
        return (n);
    }
    
    
    public Collection values() {
        List list = new ArrayList();
        Enumeration keys = request.getPropertyNames();
        while (keys.hasMoreElements()) {
            list.add(request.getProperty((String) keys.nextElement()));
        }
        return (list);
    }
    
    
    private String key(Object key) {
        if (key == null) {
            throw new IllegalArgumentException();
        } else if (key instanceof String) {
            return ((String) key);
        } else {
            return (key.toString());
        }
    }
    
    
}


class PortletRequestHeaderValuesMap extends BaseContextMap {
    
    
    public PortletRequestHeaderValuesMap(PortletRequest request) {
        this.request = request;
    }
    
    
    private PortletRequest request = null;
    
    
    public void clear() {
        throw new UnsupportedOperationException();
    }
    
    
    public boolean containsKey(Object key) {
        return (request.getProperty(key(key)) != null);
    }
    
    
    public boolean containsValue(Object value) {
        if (!(value instanceof String[])) {
            return (false);
        }
        String test[] = (String[]) value;
        Iterator values = values().iterator();
        while (values.hasNext()) {
            String actual[] = (String[]) values.next();
            if (test.length == actual.length) {
                boolean matched = true;
                for (int i = 0; i < test.length; i++) {
                    if (!test[i].equals(actual[i])) {
                        matched = false;
                        break;
                    }
                }
                if (matched) {
                    return (true);
                }
            }
        }
        return (false);
    }
    
    
    public Set entrySet() {
        Set set = new HashSet();
        Enumeration keys = request.getPropertyNames();
        while (keys.hasMoreElements()) {
            set.add(request.getProperties((String) keys.nextElement()));
        }
        return (set);
    }
    
    public boolean equals(Object o) {
        return (request.equals(o));
    }
    
    
    public Object get(Object key) {
        List list = new ArrayList();
        Enumeration values = request.getProperties(key(key));
        while (values.hasMoreElements()) {
            list.add((String) values.nextElement());
        }
        return (((String[]) list.toArray(new String[list.size()])));
    }
    
    
    public int hashCode() {
        return (request.hashCode());
    }
    
    
    public boolean isEmpty() {
        return (size() < 1);
    }
    
    
    public Set keySet() {
        Set set = new HashSet();
        Enumeration keys = request.getPropertyNames();
        while (keys.hasMoreElements()) {
            set.add(keys.nextElement());
        }
        return (set);
    }
    
    
    public Object put(Object key, Object value) {
        throw new UnsupportedOperationException();
    }
    
    
    public void putAll(Map map) {
        throw new UnsupportedOperationException();
    }
    
    
    public Object remove(Object key) {
        throw new UnsupportedOperationException();
    }
    
    
    public int size() {
        int n = 0;
        Enumeration keys = request.getPropertyNames();
        while (keys.hasMoreElements()) {
            keys.nextElement();
            n++;
        }
        return (n);
    }
    
    
    public Collection values() {
        List list = new ArrayList();
        Enumeration keys = request.getPropertyNames();
        while (keys.hasMoreElements()) {
            String key = (String) keys.nextElement();
            List list1 = new ArrayList();
            Enumeration values = request.getProperties(key);
            while (values.hasMoreElements()) {
                list1.add((String) values.nextElement());
            }
            list.add(((String[]) list1.toArray(new String[list1.size()])));
        }
        return (list);
    }
    
    
    private String key(Object key) {
        if (key == null) {
            throw new IllegalArgumentException();
        } else if (key instanceof String) {
            return ((String) key);
        } else {
            return (key.toString());
        }
    }
    
    
}


class PortletRequestMap extends BaseContextMap {
    
    
    public PortletRequestMap(PortletRequest request) {
        this.request = request;
    }
    
    
    private PortletRequest request = null;
    
    
    public void clear() {
        Iterator keys = keySet().iterator();
        while (keys.hasNext()) {
            request.removeAttribute((String) keys.next());
        }
    }
    
    
    public boolean containsKey(Object key) {
        return (request.getAttribute(key(key)) != null);
    }
    
    
    public boolean containsValue(Object value) {
        if (value == null) {
            return (false);
        }
        Enumeration keys = request.getAttributeNames();
        while (keys.hasMoreElements()) {
            Object next = request.getAttribute((String) keys.nextElement());
            if (next == value) {
                return (true);
            }
        }
        return (false);
    }
    
    public Set entrySet() {
        Set entries = new HashSet();
        for (Enumeration e = request.getAttributeNames();
        e.hasMoreElements();) {
            String key = (String) e.nextElement();
            entries.add(new Entry(key, request.getAttribute(key)));
        }
        return entries;
    }
    
    
    public boolean equals(Object o) {
        return (request.equals(o));
    }
    
    
    public Object get(Object key) {
        return (request.getAttribute(key(key)));
    }
    
    
    public int hashCode() {
        return (request.hashCode());
    }
    
    
    public boolean isEmpty() {
        return (size() < 1);
    }
    
    
    public Set keySet() {
        Set set = new HashSet();
        Enumeration keys = request.getAttributeNames();
        while (keys.hasMoreElements()) {
            set.add(keys.nextElement());
        }
        return (set);
    }
    
    
    public Object put(Object key, Object value) {
        if (value == null) {
            return (remove(key));
        }
        String skey = key(key);
        Object previous = request.getAttribute(skey);
        request.setAttribute(skey, value);
        return (previous);
    }
    
    
    public void putAll(Map map) {
        Iterator keys = map.keySet().iterator();
        while (keys.hasNext()) {
            String key = (String) keys.next();
            request.setAttribute(key, map.get(key));
        }
    }
    
    
    public Object remove(Object key) {
        String skey = key(key);
        Object previous = request.getAttribute(skey);
        request.removeAttribute(skey);
        return (previous);
    }
    
    
    public int size() {
        int n = 0;
        Enumeration keys = request.getAttributeNames();
        while (keys.hasMoreElements()) {
            keys.nextElement();
            n++;
        }
        return (n);
    }
    
    
    public Collection values() {
        List list = new ArrayList();
        Enumeration keys = request.getAttributeNames();
        while (keys.hasMoreElements()) {
            list.add(request.getAttribute((String) keys.nextElement()));
        }
        return (list);
    }
    
    
    private String key(Object key) {
        if (key == null) {
            throw new IllegalArgumentException();
        } else if (key instanceof String) {
            return ((String) key);
        } else {
            return (key.toString());
        }
    }
    
    
}


class PortletRequestParameterMap extends BaseContextMap {
    
    
    public PortletRequestParameterMap(PortletRequest request) {
        this.request = request;
    }
    
    
    private PortletRequest request = null;
    
    
    public void clear() {
        throw new UnsupportedOperationException();
    }
    
    
    public boolean containsKey(Object key) {
        return (request.getParameter(key(key)) != null);
    }
    
    
    public boolean containsValue(Object value) {
        Iterator values = values().iterator();
        while (values.hasNext()) {
            if (value.equals(values.next())) {
                return (true);
            }
        }
        return (false);
    }
    
    public Set entrySet() {
        Set entries = new HashSet();
        for (Enumeration e = request.getParameterNames();
        e.hasMoreElements();) {
            String paramName = (String) e.nextElement();
            entries.add(new Entry(paramName, request.getParameter(paramName)));
        }
        return entries;
    }
    
    
    public boolean equals(Object o) {
        return (request.equals(o));
    }
    
    
    public Object get(Object key) {
        if (key == null) {
            throw new NullPointerException();
        }
        return (request.getParameter(key(key)));
    }
    
    
    public int hashCode() {
        return (request.hashCode());
    }
    
    
    public boolean isEmpty() {
        return (size() < 1);
    }
    
    
    public Set keySet() {
        Set set = new HashSet();
        Enumeration keys = request.getParameterNames();
        while (keys.hasMoreElements()) {
            set.add(keys.nextElement());
        }
        return (set);
    }
    
    
    public Object put(Object key, Object value) {
        throw new UnsupportedOperationException();
    }
    
    
    public void putAll(Map map) {
        throw new UnsupportedOperationException();
    }
    
    
    public Object remove(Object key) {
        throw new UnsupportedOperationException();
    }
    
    
    public int size() {
        int n = 0;
        Enumeration keys = request.getParameterNames();
        while (keys.hasMoreElements()) {
            keys.nextElement();
            n++;
        }
        return (n);
    }
    
    
    public Collection values() {
        List list = new ArrayList();
        Enumeration keys = request.getParameterNames();
        while (keys.hasMoreElements()) {
            list.add(request.getParameter((String) keys.nextElement()));
        }
        return (list);
    }
    
    
    private String key(Object key) {
        if (key == null) {
            throw new IllegalArgumentException();
        } else if (key instanceof String) {
            return ((String) key);
        } else {
            return (key.toString());
        }
    }
    
    
}


class PortletRequestParameterValuesMap extends BaseContextMap {
    
    
    public PortletRequestParameterValuesMap(PortletRequest request) {
        this.request = request;
    }
    
    private PortletRequest request = null;
    
    public void clear() {
        throw new UnsupportedOperationException();
    }
    
    
    public boolean containsKey(Object key) {
        return (request.getParameter(key(key)) != null);
    }
    
    public boolean containsValue(Object value) {
        Iterator values = values().iterator();
        while (values.hasNext()) {
            if (value.equals(values.next())) {
                return (true);
            }
        }
        return (false);
    }
    
    public Object get(Object key) {
        if (key == null) {
            throw new NullPointerException();
        }
        return request.getParameterValues(key.toString());
    }
    
    
    public Set entrySet() {
        Set entries = new HashSet();
        for (Enumeration e = request.getParameterNames();
        e.hasMoreElements();) {
            String paramName = (String) e.nextElement();
            entries.add(
                    new Entry(paramName, request.getParameterValues(paramName)));
        }
        return entries;
    }
    
    
    public boolean equals(Object obj) {
        if (obj == null || !(obj instanceof PortletRequestParameterValuesMap))
            return false;
        return super.equals(obj);
    }
    
    
    public int hashCode() {
        return (request.hashCode());
    }
    
    
    public boolean isEmpty() {
        return (size() < 1);
    }
    
    
    public Set keySet() {
        Set set = new HashSet();
        Enumeration keys = request.getParameterNames();
        while (keys.hasMoreElements()) {
            set.add(keys.nextElement());
        }
        return (set);
    }
    
    
    public Object put(Object key, Object value) {
        throw new UnsupportedOperationException();
    }
    
    
    public void putAll(Map map) {
        throw new UnsupportedOperationException();
    }
    
    
    public Object remove(Object key) {
        throw new UnsupportedOperationException();
    }
    
    
    public int size() {
        int n = 0;
        Enumeration keys = request.getParameterNames();
        while (keys.hasMoreElements()) {
            keys.nextElement();
            n++;
        }
        return (n);
    }
    
    
    public Collection values() {
        List list = new ArrayList();
        Enumeration keys = request.getParameterNames();
        while (keys.hasMoreElements()) {
            list.add(request.getParameterValues((String) keys.nextElement()));
        }
        return (list);
    }
    
    
    private String key(Object key) {
        if (key == null) {
            throw new IllegalArgumentException();
        } else if (key instanceof String) {
            return ((String) key);
        } else {
            return (key.toString());
        }
    }
    
    
}


class PortletSessionMap extends BaseContextMap {
    
    
    public PortletSessionMap(PortletRequest request, PortletResponse response,
            Map requestParamMap) {
        this.session = request.getPortletSession();
        if(response instanceof ActionResponse) {
            this.namespace = (String)requestParamMap.get(ViewHandlerImpl.NAME_SPACE_PARAMETER);
        } else if (response instanceof RenderResponse){
            this.namespace = ((RenderResponse)response).getNamespace();
        } else { // May be EventResponse(used in Sun's InterPortletCommunication)
            this.namespace = "";
        }
    }
    
    private PortletSession session;
    private String namespace;
    
    public void clear() {
        Iterator keys = keySet().iterator();
        while (keys.hasNext()) {
            session.removeAttribute(namespace+(String) keys.next());
        }
    }
    
    
    public boolean containsKey(Object key) {
        return (session.getAttribute(namespace+key(key)) != null);
    }
    
    
    public boolean containsValue(Object value) {
        if (value == null) {
            return (false);
        }
        Enumeration keys =
                session.getAttributeNames(PortletSession.PORTLET_SCOPE);
        while (keys.hasMoreElements()) {
            Object next = session.getAttribute(namespace+(String) keys.nextElement());
            if (next == value) {
                return (true);
            }
        }
        return (false);
    }
    
    public Set entrySet() {
        Set entries = new HashSet();
        for (Enumeration e = session.getAttributeNames();
        e.hasMoreElements();) {
            String key = (String) e.nextElement();
            entries.add(new Entry(key, session.getAttribute(namespace+key)));
        }
        return entries;
    }
    
    
    public boolean equals(Object o) {
        return (session.equals(namespace+o));
    }
    
    
    public Object get(Object key) {
        return (session.getAttribute(namespace+key(key)));
    }
    
    
    public int hashCode() {
        return (session.hashCode());
    }
    
    
    public boolean isEmpty() {
        return (size() < 1);
    }
    
    
    public Set keySet() {
        Set set = new HashSet();
        Enumeration keys =
                session.getAttributeNames(PortletSession.PORTLET_SCOPE);
        while (keys.hasMoreElements()) {
            set.add(keys.nextElement());
        }
        return (set);
    }
    
    
    public Object put(Object key, Object value) {
        if (value == null) {
            return (remove(key));
        }
        String skey = key(key);
        Object previous = session.getAttribute(namespace+skey);
        session.setAttribute(namespace+skey, value);
        return (previous);
    }
    
    
    public void putAll(Map map) {
        Iterator keys = map.keySet().iterator();
        while (keys.hasNext()) {
            String key = (String) keys.next();
            session.setAttribute(namespace+key, map.get(key));
        }
    }
    
    
    public Object remove(Object key) {
        String skey = key(key);
        Object previous = session.getAttribute(namespace+skey);
        session.removeAttribute(namespace+skey);
        return (previous);
    }
    
    
    public int size() {
        int n = 0;
        Enumeration keys =
                session.getAttributeNames(PortletSession.PORTLET_SCOPE);
        while (keys.hasMoreElements()) {
            keys.nextElement();
            n++;
        }
        return (n);
    }
    
    
    public Collection values() {
        List list = new ArrayList();
        Enumeration keys =
                session.getAttributeNames(PortletSession.PORTLET_SCOPE);
        while (keys.hasMoreElements()) {
            list.add(session.getAttribute(namespace+(String) keys.nextElement()));
        }
        return (list);
    }
    
    
    private String key(Object key) {
        if (key == null) {
            throw new IllegalArgumentException();
        } else if (key instanceof String) {
            return ((String) key);
        } else {
            return (key.toString());
        }
    }
}

class PortletSessionWrapper implements PortletSession {
    
    public PortletSessionWrapper(PortletSession session, PortletResponse response,
            Map requestParamMap) {
        this.session = session;
        if(response instanceof ActionResponse) {
            this.namespace = (String)requestParamMap.get(ViewHandlerImpl.NAME_SPACE_PARAMETER);
        } else if (response instanceof RenderResponse){
            this.namespace = ((RenderResponse)response).getNamespace();
        } else { // May be EventResponse(used in Sun's InterPortletCommunication)
            this.namespace = "";
        }
    }
    
    private PortletSession session;
    private String namespace;
    
    public Object getAttribute(java.lang.String name)
    throws IllegalArgumentException {
        return session.getAttribute(namespace+name);
    }
    
    public Object getAttribute(String name, int scope)
    throws IllegalArgumentException {
        return session.getAttribute(namespace+name, scope);
    }
    
    
    public Enumeration getAttributeNames() {
        return session.getAttributeNames();
    }
    
    public Enumeration getAttributeNames(int scope) {
        return session.getAttributeNames(scope);
    }
    
    
    public long getCreationTime() {
        return session.getCreationTime();
    }
    
    public String getId() {
        return session.getId();
    }
    
    public long getLastAccessedTime() {
        return session.getLastAccessedTime();
    }
    
    public int getMaxInactiveInterval() {
        return session.getMaxInactiveInterval();
    }
    
    public void invalidate() {
        session.invalidate();
    }
    
    public boolean isNew() {
        return session.isNew();
    }
    
    public void removeAttribute(String name)
    throws IllegalArgumentException {
        session.removeAttribute(namespace+name);
    }
    
    public void removeAttribute(String name, int scope)
    throws IllegalArgumentException {
        session.removeAttribute(namespace+name, scope);
    }
    
    public void setAttribute(String name, Object value)
    throws IllegalArgumentException {
        session.setAttribute(namespace+name, value);
    }
    
    public void setAttribute(String name, Object value, int scope)
    throws IllegalArgumentException {
        session.setAttribute(namespace+name, value, scope);
    }
    
    public void setMaxInactiveInterval(int interval) {
        session.setMaxInactiveInterval(interval);
    }
    
    public PortletContext getPortletContext() {
        return session.getPortletContext();
    }
}

