/* 
 * CDDL HEADER START
 * The contents of this file are subject to the terms
 * of the Common Development and Distribution License 
 * (the License). You may not use this file except in
 * compliance with the License.
 *
 * You can obtain a copy of the License at
 * http://www.sun.com/cddl/cddl.html and legal/CDDLv1.0.txt
 * See the License for the specific language governing
 * permission and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL 
 * Header Notice in each file and include the License file  
 * at legal/CDDLv1.0.txt.                                                           
 * If applicable, add the following below the CDDL Header,
 * with the fields enclosed by brackets [] replaced by
 * your own identifying information: 
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Copyright 2006 Sun Microsystems Inc. All Rights Reserved
 * CDDL HEADER END
*/

package com.sun.faces.portlet;


import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.faces.FacesException;
import javax.faces.application.FacesMessage;
import javax.faces.application.ViewHandler;
import javax.faces.component.UIComponent;
import javax.faces.component.UIViewRoot;
import javax.faces.context.FacesContext;
import javax.faces.el.ValueBinding;
import javax.faces.event.PhaseEvent;
import javax.faces.event.PhaseId;
import javax.faces.event.PhaseListener;
import javax.faces.lifecycle.Lifecycle;
import javax.faces.render.RenderKitFactory;

import javax.portlet.PortletSession;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


/**
 * <p>Custom implementation of <code>Lifecycle</code> that implements a
 * portlet-specific <code>Lifecycle</code>.</p>
 */

public final class LifecycleImpl extends Lifecycle {


    // ------------------------------------------------------------ Constructors


    /**
     * <p>Construct a new <code>LifecycleImpl</code> instance.</p>
     */
    public LifecycleImpl() {
        super();
        if (log.isTraceEnabled()) {
	    log.trace("Created Lifecycle " + this);
	}
    }


    // -------------------------------------------------------- Static Variables


    /**
     * <p><code>Log</code> instance for this class.</p>
     */
    private static final Log log = LogFactory.getLog(LifecycleImpl.class);

    /**
     * <p>Portlet initialization parameter which holds the value of one of
     * initial mode identifier. i.e either INIT_VIEW_PARAMETER or
     * INIT_EDIT_PARAMETER or INIT_HELP_PARAMETER</p>
     */
    public static final String INIT_PARAMETER =
        "com.sun.faces.portlet.INIT";

    /**
     * <p>Portlet initialization parameter under which the application
     * may specify an initial view mode identifier to be displayed.</p>
     */
    public static final String INIT_VIEW_PARAMETER =
        "com.sun.faces.portlet.INIT_VIEW";

    /**
      * <p>Portlet initialization parameter under which the application
      * may specify an initial edit mode identifier to be displayed.</p>
      */
     public static final String INIT_EDIT_PARAMETER =
         "com.sun.faces.portlet.INIT_EDIT";

     /**
      * <p>Portlet initialization parameter under which the application
      * may specify an initial help mode identifier to be displayed.</p>
      */
     public static final String INIT_HELP_PARAMETER =
         "com.sun.faces.portlet.INIT_HELP";

    /**
     * <p>Portlet session attribute (in portlet scope) under which we will
     * save the window state identifer. i.e one of the WINDOW_STATE_ATTR_* values.</p>
     */
    private static final String PREVIOUS_WINDOW_STATE =
        "com.sun.faces.portlet.PREVIOUS_WINDOW_STATE";

    /**
     * <p>Portlet session attribute (in portlet scope) under which we will
     * save state information for VIEW for the current window.</p>
     */
    private static final String WINDOW_STATE_ATTR_VIEW =
        "com.sun.faces.portlet.WINDOW_STATE_VIEW";

    /**
     * <p>Portlet session attribute (in portlet scope) under which we will
     * save state information for EDIT for the current window.</p>
     */
    private static final String WINDOW_STATE_ATTR_EDIT =
        "com.sun.faces.portlet.WINDOW_STATE_EDIT";

    /**
     * <p>Portlet session attribute (in portlet scope) under which we will
     * save state information for HELP for the current window.</p>
     */
    private static final String WINDOW_STATE_ATTR_HELP =
        "com.sun.faces.portlet.WINDOW_STATE_HELP";


    /**
     * <p>Portlet session attribute (in portlet scope) under which we will
     * save a Boolean TRUE or FALSE for the current window.</p>
     */
    public static final String ERROR_FLAG =
        "com.sun.faces.portlet.ERROR_FLAG";


    // ------------------------------------------------------ Instance Variables


    /**
     * <p>The set of <code>PhaseListener</code>s registered with this
     * <code>Lifecycle</code> instance, in order of registration.</p>
     */
    private List listeners = new ArrayList();


    /**
     * <p>The set of {@link Phase} instances that are to be
     * executed by the <code>execute()</code> method, in order by the
     * ordinal property of each phase.</p>
     */
    private Phase phases[] = {
        null, // ANY_PHASE holder
        new RestoreViewPhase(),
        new ApplyRequestValuesPhase(),
        new ProcessValidationsPhase(),
        new UpdateModelValuesPhase(),
        new InvokeApplicationPhase()
    };


    /**
     * <p>The {@link Phase} instance to process Render Response phase.</p>
     */
    private Phase response = new RenderResponsePhase();


    // ------------------------------------------------------- Lifecycle Methods


    // Add a new PhaseListener to the set of registered listeners
    public void addPhaseListener(PhaseListener listener) {

        if (listener == null) {
	    throw new NullPointerException();
        }
        if (log.isDebugEnabled()) {
            log.debug("addPhaseListener(" + listener.getPhaseId().toString()
                      + "," + listener);
        }
        synchronized (listeners) {
            listeners.add(listener);
        }

    }


    // Execute the phases up to but not including Render Response
    public void execute(FacesContext context) throws FacesException {

        if (context == null) {
	    throw new NullPointerException();
        }

        if (log.isDebugEnabled()) {
            log.debug("execute(" + context + ")");
        }

        restore(context, true);

        setErrorFlag(context, Boolean.FALSE);

        for (int i = 1; i < phases.length; i++) {
            PhaseId phaseId = (PhaseId) PhaseId.VALUES.get(i);
            if (context.getRenderResponse() ||
                context.getResponseComplete()) {
                /*
                    (a) If there are any validation/conversion errors, then its an error.
                    (b) If there are no validation/conversion errors, it means either
                    immediate=true is specified or FacesContext.renderResponse() has been
                    called.
                */
                Iterator itr = context.getMessages();
                if(itr.hasNext()) {
                    setErrorFlag(context, Boolean.TRUE);
                }
                break;
            }
            phase(phaseId, phases[i], context);
        }

        save(context, true);

    }


    // Return the set of PhaseListeners that have been registered
    public PhaseListener[] getPhaseListeners() {

        synchronized (listeners) {
            PhaseListener results[] = new PhaseListener[listeners.size()];
            return ((PhaseListener[]) listeners.toArray(results));
        }

    }


    // Execute the Render Response phase
    public void render(FacesContext context) throws FacesException {
        if (context == null) {
	    throw new NullPointerException();
        }

        if (log.isDebugEnabled()) {
            log.debug("render(" + context + ")");
        }

        restore(context, false);

        setErrorFlag(context, Boolean.FALSE);

        if (!context.getResponseComplete()) {
            phase(PhaseId.RENDER_RESPONSE, response, context);
        }
        
        save(context, false);
    }


    // Remove a registered PhaseListener from the set of registered listeners
    public void removePhaseListener(PhaseListener listener) {

        if (listener == null) {
	    throw new NullPointerException();
        }
        if (log.isDebugEnabled()) {
            log.debug("removePhaseListener(" + listener.getPhaseId().toString()
                      + "," + listener);
        }
        synchronized (listeners) {
            listeners.remove(listener);
        }

    }


    // --------------------------------------------------------- Private Methods


    /**
     * <p>Execute the specified phase instance for the current request.</p>
     *
     * @param phaseId Phase identifier of the current phase
     * @param phase {@link Phase} instance to execute
     * @param context <code>FacesContext</code> for the current request
     *
     * @exception FacesException if thrown by the phase execution
     */
    private void phase(PhaseId phaseId, Phase phase, FacesContext context)
        throws FacesException {

        boolean exceptionThrownInRender = false;
        Exception tempException = null;

        if (log.isTraceEnabled()) {
            log.trace("phase(" + phaseId.toString() + "," + context + ")");
        }

        // Acquire (once) the list of interested phase listeners
        // Clone it so that we are independent of changes during the
        // execution of this phase
        PhaseListener listeners[] = getPhaseListeners();

        try {

        // Notify the "beforePhase" method of interested listeners
            if (listeners.length > 0) {
                PhaseEvent event = new PhaseEvent(context, phaseId, this);
                for (int i = 0; i < listeners.length; i++) {
                    if (phaseId.equals(listeners[i].getPhaseId()) ||
                        PhaseId.ANY_PHASE.equals(listeners[i].getPhaseId())) {
                        listeners[i].beforePhase(event);
                    }
                }
            }

        } catch (Exception e) {

            // Log the problem, but continue
            if (log.isDebugEnabled()) {
                log.debug("beforePhase(" + phaseId.toString() + "," + context +
                          ") threw exception", e);
            }

        }

        try {

            try {

                // Execute this phase itself
                phase.execute(context);

            } catch (Exception e) {

                // Log the problem, but continue
                if (log.isDebugEnabled()) {
                    log.debug("executePhase(" + phaseId.toString() + "," + context +
                              ") threw exception", e);
                }

                // Don't rethrow as it will break the jsf lifecycle execution
                // If exception is thrown in the render phase, hold a reference to the exception
                if(phaseId.compareTo(PhaseId.RENDER_RESPONSE) == 0) {
                    exceptionThrownInRender = true;
                    tempException = e;
                }
            }

        // Notify the "afterPhase" method of interested listeners
            if (listeners.length > 0) {
                PhaseEvent event = new PhaseEvent(context, phaseId, this);
                for (int i = listeners.length - 1; i >= 0; i--) {
                    if (phaseId.equals(listeners[i].getPhaseId()) ||
                        PhaseId.ANY_PHASE.equals(listeners[i].getPhaseId())) {
                        listeners[i].afterPhase(event);
                    }
                }
            }

        } catch (Exception e) {

            // Log the problem, but continue
            if (log.isDebugEnabled()) {
                log.debug("afterPhase(" + phaseId.toString() + "," + context +
                          ") threw exception", e);
            }

        }

        if(exceptionThrownInRender)
            throw new FacesException(tempException);
    }


    /**
     * <p>Restore state information for the current window into the
     * specified <code>FacesContext</code> instance.</p>
     *
     * @param context <code>FacesContext</code> for this request
     * @param action Flag indicating this is action mode (true)
     *  or render mode (false)
     */
    private void restore(FacesContext context, boolean action) {
        if (log.isTraceEnabled()) {
            log.trace("restore(" + context + "," + action + ")");
        }
        
        // Retrieve the cached state information (if any)
        String windowStateIdentifier = getWindowStateIdentifier((String) context.getExternalContext().
                                getRequestMap().get(INIT_PARAMETER));

    	PortletSession session = (PortletSession)context.getExternalContext().getSession(true);

    	String previousWindowStateIndentifier = (String)session.getAttribute(PREVIOUS_WINDOW_STATE);

        FacesPortletState state = null;
        // This will ensure that the state information saved for INIT_VIEW will be restored
        // only for INIT_VIEW and so on..
        if(previousWindowStateIndentifier != null
                        && previousWindowStateIndentifier.equals(windowStateIdentifier)) {
            state = (FacesPortletState)session.getAttribute(windowStateIdentifier);

        }

        if (state == null) {
            setViewId(context);
            return;
        }

        // Restore the cached state information
        if (!action) {
            Iterator messages;
            Iterator clientIds = state.getClientIds();
            while (clientIds.hasNext()) {
                String clientId = (String) clientIds.next();
                messages = state.getMessages(clientId);
                while (messages.hasNext()) {
                    context.addMessage(clientId, (FacesMessage) messages.next());
                }
            }
        }
        context.setViewRoot(state.getViewRoot());

        // Remove the cached state information
        session.removeAttribute(windowStateIdentifier);

        if (log.isTraceEnabled()) {
            log.trace("End restore()");
        }

    }


    /**
     * <p>Save state information for the current window from the
     * specified <code>FacesContext</code> instance.</p>
     *
     * @param context <code>FacesContext</code> for this request
     * @param action Flag indicating this is action mode (true)
     *  or render mode (false)
     */
    private void save(FacesContext context, boolean action) {
        if (log.isTraceEnabled()) {
            log.trace("save(" + context + "," + action + ")");
        }
        // Save state information from this FacesContext
        FacesPortletState state = new FacesPortletState();
        Iterator messages;
        Iterator clientIds = context.getClientIdsWithMessages();
        while (clientIds.hasNext()) {
            String clientId = (String) clientIds.next();
            messages = context.getMessages(clientId);
            while (messages.hasNext()) {
                state.addMessage(clientId, (FacesMessage) messages.next());
            }
            // Log the message
            if (log.isErrorEnabled()) {
                log.error(clientId + state.getMessagesBuffer(clientId));
            }
        }
        state.setViewRoot(context.getViewRoot());

        // Cache the state information in a session in portlet scope
        String windowStateIdentifier = getWindowStateIdentifier((String) context.getExternalContext().
                                getRequestMap().get(INIT_PARAMETER));

    	PortletSession session = (PortletSession)context.getExternalContext().getSession(true);
        // Save the window state identifier in a session in portlet scope
        // This information is used during restore to restore the cached information for the
        // appropriate window state. This will ensure that the state information saved for
        // INIT_VIEW will be restored only for INIT_VIEW and so on..
        session.setAttribute(PREVIOUS_WINDOW_STATE, windowStateIdentifier);

        session.setAttribute(windowStateIdentifier, state);
    }


    /**
     * <p>Get the view identifier to a default page specified in a
     * portlet init parameter.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     */
    private String getInitViewId(FacesContext context) {
        Map requestMap = context.getExternalContext().getRequestMap();
        String initParameterIdentifier = (String) requestMap.get(INIT_PARAMETER);
        String viewId = (String) requestMap.get(initParameterIdentifier);
        return viewId;
    }

    /**
     * <p>Set the view identifier to a default page specified in a
     * portlet init parameter.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     */
    private void setViewId(FacesContext context) {
        String viewId = getInitViewId(context);

        if (context.getViewRoot() == null) {
            context.setViewRoot(context.getApplication().getViewHandler().createView(context, viewId));
            if (log.isDebugEnabled()) {
                log.debug("Created new ViewRoot with View Id" + context.getViewRoot().getViewId());
            }
        } else {
            context.getViewRoot().setViewId(viewId); 
            if (log.isDebugEnabled()) {
                log.debug("set viewId to " + viewId);
            }
        }
        
        context.getViewRoot().setRenderKitId(RenderKitFactory.HTML_BASIC_RENDER_KIT);
    }

    /**
     * Returns the window state attribute identifier based on the
     * init parameter identifier.
     *
     * @param initParameterIdentifier the initialization parameter identifier
     * @return window state attribute identifier
     */
    private String getWindowStateIdentifier(String initParameterIdentifier) {
        if(initParameterIdentifier.equals(INIT_VIEW_PARAMETER))
            return WINDOW_STATE_ATTR_VIEW;
        else if(initParameterIdentifier.equals(INIT_EDIT_PARAMETER))
            return WINDOW_STATE_ATTR_EDIT;
        else if(initParameterIdentifier.equals(INIT_HELP_PARAMETER))
            return WINDOW_STATE_ATTR_HELP;
        else
            return null;

    }

    private void setErrorFlag(FacesContext context, Boolean value) {
    	PortletSession session = (PortletSession)context.getExternalContext().getSession(true);
    	session.setAttribute(ERROR_FLAG, value);
    }

// ------------------------------------------------------- Phase Implementations


    interface Phase {

        public void execute(FacesContext context)
            throws FacesException;

    }


    final class RestoreViewPhase implements Phase {

        public void execute(FacesContext context) throws FacesException {
            if (log.isDebugEnabled()) {
                log.debug("Begin RestoreViewPhase");
            }
            // get the viewId from the context
            String viewId = context.getViewRoot().getViewId();
            if (viewId == null) {
                if (log.isTraceEnabled()) {
                    log.trace("No view identifier found");
                }
                throw new FacesException // PENDING - i18n
                    ("No view identifier in this request");
            }

            // Try to restore the view
            ViewHandler vh = context.getApplication().getViewHandler();
            UIViewRoot viewRoot = vh.restoreView(context, viewId);
            if (viewRoot == null) {
                if (log.isTraceEnabled()) {
                    log.trace("Creating new view '" + viewId + "'");
                }
                viewRoot = vh.createView(context, viewId);
                context.renderResponse();
            } else {
                if (log.isTraceEnabled()) {
                    log.trace("Restoring old view '" + viewId + "'");
                }
            }
            context.setViewRoot(viewRoot);
            doPerComponentActions(context, viewRoot);
            if (log.isDebugEnabled()) {
                log.debug("End RestoreViewPhase");
            }

        }


        // Do the per-component actions needed during restore
        private void doPerComponentActions
            (FacesContext context, UIComponent component) {

            Iterator kids = component.getFacetsAndChildren();
            while (kids.hasNext()) {
                doPerComponentActions(context, (UIComponent) kids.next());
            }
            ValueBinding vb = component.getValueBinding("binding");
            if (vb != null) {
                vb.setValue(context, component);
            }
        }


    }


    final class ApplyRequestValuesPhase implements Phase {

        public void execute(FacesContext context) throws FacesException {
            if (log.isDebugEnabled()) {
                log.debug("Begin ApplyRequestValuesPhase");
            }
            UIViewRoot viewRoot = context.getViewRoot();
            try {
                viewRoot.processDecodes(context);
            } catch (FacesException e) {
                log.error("" + e, e);
                throw e;
            } catch (RuntimeException e) {
                log.error("" + e, e);
                throw new FacesException(e);
            }
            if (log.isDebugEnabled()) {
                log.debug("End ApplyRequestValuesPhase");
            }
        }

    }


    final class ProcessValidationsPhase implements Phase {

        public void execute(FacesContext context) throws FacesException {
            if (log.isDebugEnabled()) {
                log.debug("Begin ProcessValidationsPhase");
            }
            UIViewRoot viewRoot = context.getViewRoot();
            try {
                viewRoot.processValidators(context);
            } catch (FacesException e) {
                log.error("" + e, e);
                throw e;
            } catch (RuntimeException e) {
                log.error("" + e, e);
                throw new FacesException(e);
            }
            if (log.isDebugEnabled()) {
                log.debug("End ProcessValidationsPhase");
            }
        }

    }


    final class UpdateModelValuesPhase implements Phase {

        public void execute(FacesContext context) throws FacesException {
            if (log.isDebugEnabled()) {
                log.debug("Begin UpdateModelValuesPhase");
            }
            UIViewRoot viewRoot = context.getViewRoot();
            try {
            	viewRoot.processUpdates(context);
            } catch (FacesException e) {
                log.error("" + e, e);
                throw e;
            } catch (RuntimeException e) {
                log.error("" + e, e);
                throw new FacesException(e);
            }
            if (log.isDebugEnabled()) {
                log.debug("End UpdateModelValuesPhase");
            }
        }

    }


    final class InvokeApplicationPhase implements Phase {

        public void execute(FacesContext context) throws FacesException {
            if (log.isDebugEnabled()) {
                log.debug("Begin InvokeApplicationPhase");
            }
            UIViewRoot viewRoot = context.getViewRoot();
            try {
            	viewRoot.processApplication(context);
            } catch (FacesException e) {
                log.error("" + e, e);
                throw e;
            } catch (RuntimeException e) {
                log.error("" + e, e);
                throw new FacesException(e);
            }
            if (log.isDebugEnabled()) {
                log.debug("End InvokeApplicationPhase");
            }
        }

    }


    final class RenderResponsePhase implements Phase {

        public void execute(FacesContext context) throws FacesException {
            if (log.isDebugEnabled()) {
                log.debug("Begin RenderResponsePhase");
            }
            String requestURI = context.getViewRoot().getViewId();
            if (log.isDebugEnabled()) {
                log.debug("About to render view " + requestURI);
            }
            try {
                context.getApplication().getViewHandler().
                				renderView(context, context.getViewRoot());
            } catch (FacesException e) {
                log.error("" + e, e);
                throw e;
            } catch (IOException e) {
                throw new FacesException(e);
            }
            if (log.isDebugEnabled()) {
                log.debug("End RenderResponsePhase");
            }
        }

    }


}


// ------------------------------------------------------------- Private Classes


/**
 * <p>Private class to represent the JavaServer Faces specific information
 * that is saved and restored for a particular window.
 */

final class FacesPortletState {


    // Methods Saving and Restoring Messages
    private Map messages = new HashMap(); // key=clientId, value=List of FacesMessage
    public void addMessage(String clientId, FacesMessage message) {
        List list = (List) messages.get(clientId);
        if (list == null) {
            list = new ArrayList();
            messages.put(clientId, list);
        }
        list.add(message);
    }
    public Iterator getMessages(String clientId) {
        List list = (List) messages.get(clientId);
        if (list != null) {
            return (list.iterator());
        } else {
            return (Collections.EMPTY_LIST.iterator());
        }
    }

    public StringBuffer getMessagesBuffer(String clientId) {
        List list = (List) messages.get(clientId);
        StringBuffer buffer = new StringBuffer();
        if (list != null) {
            Iterator messages = list.iterator();
            FacesMessage message;
            while (messages.hasNext()) {
            	message = (FacesMessage) messages.next();
            	buffer.append(" ");
            	buffer.append(message.getDetail());
            }
        }
        return buffer;
    }

    // Iterate over the client ids in this view
    public Iterator getClientIds() {
        return (messages.keySet().iterator());
    }

    // The UIViewRoot that is the root of our component tree
    private UIViewRoot viewRoot;
    public UIViewRoot getViewRoot() { return this.viewRoot; }
    public void setViewRoot(UIViewRoot viewRoot)
    { this.viewRoot = viewRoot; }

}
