/*
 * CDDL HEADER START
 * The contents of this file are subject to the terms
 * of the Common Development and Distribution License
 * (the License). You may not use this file except in
 * compliance with the License.
 *
 * You can obtain a copy of the License at
 * http://www.sun.com/cddl/cddl.html and legal/CDDLv1.0.txt
 * See the License for the specific language governing
 * permission and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL
 * Header Notice in each file and include the License file
 * at legal/CDDLv1.0.txt.
 * If applicable, add the following below the CDDL Header,
 * with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Copyright 2006 Sun Microsystems Inc. All Rights Reserved
 * CDDL HEADER END
*/

package com.sun.faces.portlet;


import java.io.IOException;
import java.util.Locale;

import javax.faces.application.ViewHandler;
import javax.faces.component.UIViewRoot;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.FacesException;

import javax.portlet.PortletURL;
import javax.portlet.RenderResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


/**
 * <p>Concrete implementation of <code>ViewHandler</code> for use in
 * a portlet environment.  This implementation delegates to the
 * standard <codeViewHandler</code> instance provided to our constructor,
 * and only implements portlet-specific behavior where necessary.</p>
 */

public final class ViewHandlerImpl extends ViewHandler {


    // ------------------------------------------------------------- Constructor


    /**
     * <p>Construct a new <code>ViewHandler</code> instance that delegates
     * all non-portlet-specific behavior to the specified implementation.
     *
     * @param handler The <code>ViewHandler</code> instance to whom
     *  we can delegate
     *
     * @exception NullPointerException if <code>handler</code>
     *  is <code>null</code>
     */
    public ViewHandlerImpl(ViewHandler handler) {

        if (handler == null) {
            throw new NullPointerException();
        }
        if (log.isDebugEnabled()) {
            log.debug("Delegating to '" + handler + "'");
        }
        this.handler = handler;

    }


    // -------------------------------------------------------- Static Variables


    /**
     * <p>The URL parameter we use to pass the view identifier of the
     * requested view.</p>
     */
    public static final String VIEW_ID_PARAMETER =
        "com.sun.faces.portlet.VIEW_ID";

    /**
     * <p>The URL parameter we use to pass the namespace of the portlet.</p>
     */
    public static final String NAME_SPACE_PARAMETER =
        "com.sun.faces.portlet.NAME_SPACE";

    // The Log instance for this class
    private static final Log log = LogFactory.getLog(ViewHandlerImpl.class);


    // ------------------------------------------------------ Instance Variables


    // The ViewHandler we delegate to
    private ViewHandler handler;


    // ----------------------------------------------------- ViewHandler Methods


    public Locale calculateLocale(FacesContext context) {
        return (handler.calculateLocale(context));
    }


    public String calculateRenderKitId(FacesContext context) {
        return (handler.calculateRenderKitId(context));
    }


    public UIViewRoot createView(FacesContext context, String viewId) {
        return (handler.createView(context, viewId));
    }


    public String getActionURL(FacesContext context, String viewId) {
        Object r = context.getExternalContext().getResponse();
        if (!(r instanceof RenderResponse)) {
            if (log.isDebugEnabled()) {
                log.debug("Must be a RenderResponse");
            }
            throw new IllegalStateException("Must be a RenderResponse");
        }
        RenderResponse response = (RenderResponse) r;
        PortletURL actionURL = response.createActionURL();
        actionURL.setParameter(VIEW_ID_PARAMETER, viewId);

        //Start Julien's patchc
		   // A unique value is needed while saving values in the session.
		        // As the render response namespace is unique, it is used.
		        actionURL.setParameter(NAME_SPACE_PARAMETER, response.getNamespace());
		//End patch

        if (log.isTraceEnabled()) {
    	    log.trace("Action URL:" + actionURL.toString());
        }
        return (actionURL.toString());
    }


    public String getResourceURL(FacesContext context, String path) {
        return (handler.getResourceURL(context, path));
    }


    public void renderView(FacesContext context, UIViewRoot viewToRender)
        throws IOException, FacesException {
    		handler.renderView(context, viewToRender);
    }


    public UIViewRoot restoreView(FacesContext context, String viewId) {
        return (handler.restoreView(context, viewId));
    }


    public void writeState(FacesContext context) throws IOException {
        handler.writeState(context);
        // A unique value is needed while saving values in the session.
        // As the render response namespace is unique, we write the namespace obtainer
        // from the render response as hidden field and send it with the page.
        ResponseWriter writer = context.getResponseWriter();
        RenderResponse response = (RenderResponse)context.getExternalContext().getResponse();
        writer.startElement("input", context.getViewRoot());
        writer.writeAttribute("type", "hidden", null);
        writer.writeAttribute("name", NAME_SPACE_PARAMETER, null);
        writer.writeAttribute("id", NAME_SPACE_PARAMETER, null);
        writer.writeAttribute("value", response.getNamespace(), null);
    }
}
